<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Http\Repositories\MoviesRepository;
use App\Models\Movies;
use App\Http\Services\MoviesService;
use Laracasts\Flash\Flash;
use App\Http\Traits\ResponseTrait;
use App\Http\Requests\MoviesRequest;

class MoviesController extends Controller
{
    use ResponseTrait;

    protected $moviesRepository;

    protected $moviesService;

    public function __construct(MoviesRepository $moviesRepository, MoviesService $moviesService)
    {
        $this->moviesRepository = $moviesRepository;

        $this->moviesService = $moviesService;
    }

    public function index() 
    {
        $movies = $this->moviesRepository->findAll();

        return view('admin.movies.index', compact('movies'));
    }

    public function create(Movies $movies)
    {
        $data = $this->moviesRepository->getFormData($movies);

        return view('admin.movies.form', $data);
    }

    public function store(MoviesRequest $request)
    {
        $data = $request->only('name', 'released_date', 'lang', 'duration', 'description', 'streaming_link', 'ordering', 'status');
        if($request->genres) {
            $data['genres'] = $request->genres;
        }

        if($movies = $this->moviesService->create($data)) {
            $this->moviesService->createUpdateGenres($request, $movies);
            $destinationPath = public_path() . config('dashboard.movie_poster_image');
            $destinationCoverPath = public_path() . config('dashboard.movie_cover_image');

            if ($fileName = isFileExist($request, $movies, 'poster_image', $destinationPath)) {
                $movies->poster_image = $fileName;
                $movies->update();
            }

            if ($fileName = isFileExist($request, $movies, 'cover_image', $destinationCoverPath)) {
                $movies->cover_image = $fileName;
                $movies->update();
            }

            Flash::success($this->success());
        }
        else {
            Flash::error($this->tryAgain());
        }

        return redirect()->route('admin.movies.index');
    }

    public function edit($id)
    {
        $movies = $this->moviesRepository->find($id);

        $data = $this->moviesRepository->getFormData($movies);

        $data['genres'] = $this->moviesRepository->genresToArray($movies->genres);

        return view('admin.movies.form', $data);
    }

    public function update(MoviesRequest $request, $id)
    {
        $movies = $this->moviesRepository->find($id);
        
        $data = $request->only('name', 'released_date', 'lang', 'duration', 'description', 'streaming_link', 'ordering', 'status');
        if($request->genres) {
            $data['genres'] = $request->genres;
        }
        
        if($moviesData = $this->moviesService->update($movies, $data)) {
            $this->moviesService->createUpdateGenres($request, $movies);
            $posterImage = $this->uploadPosterImage($request, $movies);
            $coverImage = $this->uploadCoverImage($request, $movies);
            Movies::where('id', $movies->id)->update(['poster_image' => $posterImage, 'cover_image' => $coverImage]);
            
            Flash::success($this->updated());
        }
        else {
            Flash::error($this->tryAgain());
        }

        return redirect()->route('admin.movies.index');

    }

    public function destroy($id)
    {
        $movies = $this->moviesRepository->find($id);

        if($this->moviesService->delete($movies)) {

            Flash::success($this->deleted());
        }
        else {
            Flash::error($this->tryAgain());
        }

        return redirect()->back();
    }

    public function uploadPosterImage($request, $movies) 
    {
        if(isset($request->poster_file)) {
            $destinationPath = public_path() . config('dashboard.movie_poster_image');
            
            if ($fileName = isFileExist($request, $movies, 'poster_file', $destinationPath)) {
                $posterImage = baseUrl(). config('dashboard.movie_poster_image') .$fileName;
            }
            else {
                $posterImage = $request->poster_image;
            }
        }
        else {
            $posterImage = $request->poster_image;
        }

        return $posterImage;
    }

    public function uploadCoverImage($request, $movies)
    {
        if(isset($request->cover_file)) {
            $destinationPath = public_path() . config('dashboard.movie_cover_image');
            if ($fileName = isFileExist($request, $movies, 'cover_file', $destinationPath)) {
                $coverImage = baseUrl(). config('dashboard.movie_cover_image') .$fileName;
            }
            else {
                $coverImage = $request->cover_image;
            }
        }
        else {
            $coverImage = $request->cover_image;
        }

        return $coverImage;

        

    }



}
    